% Version made available on Website   MAY 2005 LH Carney

% Retinalgrowth.m  Model of radial retinal growth within the INL plane.
% This program calls another file, "growth.m" on each 'loop' of the simulation. In each loop,
% a new layer of undifferentiated cells is added to the MGZ.
% Three spatial statistical analyses of seed and model patterns are performed:
% Nearest Neighbor Distance Analysis (Cook, 1996)
% Density Recovery Profile Analysis (Rodieck, 1991)
% Quadrat Analysis (Grieg-Smith, 1964)

% %***********  IMPORTANT VARIABLES   **********************************
% 
% lambdaI_over_NND: space constant for the decaying exponential signal
% threshold:        sets the lower limits for the summed inhibitory signal above which a
%                       precursor cell will differentiate
% mgz:              radial width of the mitotic germinal zone in um
% undiff_density:   the density of mgz precursor cells in cells/mm^2
%
% In growth.m:
% lsd % Standard deviation of Gaussian distribution used to choose lambda_I for each differentiated cell
% a1  % Amplitude value of signal at source cell (i.e., at coordinates of each differentiated cell)
% %*********************************************************************

clear all
close all

lambdaI_over_NND = 1.00; 
threshold = -1.;

disp('Input seed data file must have 2 columns (x and y cell positions) in microns');
filename = input('Filenum to analyze (e.g. IPCsdemo.txt [in single quotes]): ');
eval(['!copy ' filename ' tmp_file']); % copy the file of interest into a tmp file 
load tmp_file; % load the file
for(i = 1:length(tmp_file(:,1)))  % Read in Seed Cell coordinates (these are the initial differentiated cells)
    x(i) = tmp_file(i,1);
    y(i) = tmp_file(i,2);
end
n = length(x); % total # of cells

%Find and plot analysis areas 
r = 0.9 * (max(y) - min(y));  % radius over which pattern will be analyzed (0.9 makes sure that box doesn't extend beyond data, and allows for uneven border)
el = r; % 'el' is the length of one side of the analysis box

% find middle of pattern, and boundaries of analysis rectangle 
xmid = (max(x) - min(x))/2 + min(x);  
ymid = (max(y) - min(y))/2 + min(y);
xlo = xmid - el/2; xhi = xmid + el/2;  
ylo = ymid - el/2; yhi = ymid + el/2;

%%%%%%%%%%%%%%%% NND FOR SEED PATTERN %%%%%%%%%%%%%%%%%
N = 0;
for i = 1:n % go through cells in analysis box (this avoids edge effects for model data)
    if( (x(i) <= xhi) & (x(i) >= xlo) & (y(i) <= yhi) & (y(i) >= ylo) )
        N = N+1;
        dist = sqrt(((x - x(i)) .^ 2) + ((y - y(i)) .^ 2));
        dist = sort(dist);
        nnds(N) = dist(2); % skip 'mimimum' distance of 0 (distance to self)
        xref(N) = x(i);  % this array is used in DRP analysis below
        yref(N)= y(i);   %     ditto
    end
end
% Now, N = # of cells in the analysis box - this is used for Quadrat analysis below
NND = mean(nnds);

%%%%%%%%%%%%%%% CENTER AND TRIM SEED PATTERN %%%%%%%%%%

% Center the pattern at 0 so it can be 'trimmed' to have a smooth outer boundary (before adding the MGZ) 
% Center the seed pattern
x = x - xmid; 
y = y - ymid;

% Now, Convert to polar coordinates, and then "trim" the pattern, so outer boundary will be smooth
rho = sqrt([(x.^2)+(y.^2)]);
theta = atan2(y,x); 
% ... and trim the pattern
j=1;
for i = 1:length(rho)
    if rho(i) <= (9./10.) * max(rho); % trim to radius of 9/10 of max to smooth out the outermost edge of seed (Mel)    
        %rho(i) <= (3./4.) * max(rho); % trim to radius of 3/4 of max to smooth out the outermost edge of seed (LHC)
        Drho(j) = rho(i);
        Dtheta(j) = theta(i);
        j = j+1;
    end
end
Drho = Drho';    %convert line vectors to column vectors
Dtheta = Dtheta';

num_seed = length(Drho); % number of cells in the trimmed seed pattern
clear i j rho theta x y 

%%%%%%%%%%%%%% ADD LAYER OF PRECURSOR CELLS (MGZ)%%%%%%%%%
%Add 0.90 scalar to rhomax (?) - this would avoid the gap between seed and newly differentiated cells
rhomax = 0.90 * max(Drho);      % initially: MAX value in (trimmed) data seed for these amacrine cells - this will get updated after each loop
init_rhomax = rhomax;           % allows you to keep track of radial distance at which growth is initiated
undiff_density = 3 * 1700; % cells/mm^2;  NOTE - This is ~3 * density of PKC cells
thetamin = 30.; % specify angular range over which model will add differentiated cells
thetamax = 150.; 
thetarange = 2 * pi * (thetamax - thetamin)/360.;

mgz = 50.; % width of MGZ in micrometers
nloops = floor(2.3 * rhomax/mgz)  % set nloops of growth to some value of the original radial distance of seed pattern
if (nloops == 0) nloops = 1; end  % be sure that at least one layer of mgz gets added to seed

for loop = 1:nloops % These are "growth loops" for mgz (essentially, these are 'MGZ clock tics')
    
    % Calculate the number of precursor cells in the mgz
    arclength = thetarange * rhomax;   
    area = mgz * (arclength * 1e-6);
    numUD = floor(area * undiff_density); % # of new undiff cells to add in MGZ layer
    radiusUD = mgz*rand([1,numUD]) + rhomax;  % specify random undiff cell positions
    thetaUD  = 2 * pi * thetamin/360. + thetarange * rand([1,numUD]);       
    
    if (loop > 1)   % Hold on to 'leftover undiff'ed cells - skip this on first loop
        theta_leftover = atan2(Uy,Ux);
        radius_leftover = sqrt([(Ux.^2) + (Uy.^2)]);
        thetaUD = [theta_leftover'; thetaUD']'; % combine 'leftover' undiffs with new ones
        radiusUD = [radius_leftover'; radiusUD']';  
    end
    
    undiffs = [thetaUD radiusUD]; %;        % now place all undiff'ed cells into this matrix (undiffs has 2 columns)
    diffAm = [Dtheta Drho];                 % DIFFERENTIATED CELL MATRIX
    
    rhomax = rhomax + mgz;                  % increment the 'outer rim' of the mgz after each loop
    
    
    %%%%%%%%%%%% REMOVE CLOSELY SPACED PRECURSOR CELLS (too close to be physically possible)
    spacelim = 2;    %This is the distance in microns allowable between centers of precursor cells
    %This stops the occurence of almost coincidal cells, on top of each other
    
    % Convert to rectangular coordinates to make distance calculations simpler
    mux = (radiusUD.*cos(thetaUD));
    muy = (radiusUD.*sin(thetaUD));
    clear thetaUD, radiusUD;
    
    
    index1 = 1;   % a counter for ikill(index1)
    %Calculate distance between undiff'ed cells and remove those closer together than 'spacelim' value
    %Takes care not to find distance between same 2 cells twice, and  disregards 0 if same cell is used
    ikill = []; % 'initialize' this, such that even if no cells are eliminated, the length(ikill) call below still works
    for j = 1: (length(mux)-1)
        for k = j+1:length(mux)
            mdistance = sqrt([(mux(j) - mux(k))^2 + (muy(j) - muy(k))^2]); % was mdistance(j,k) [huge arrays]
            if mdistance < spacelim & mdistance > 0          
                ikill(index1) = j;   %This keeps track of what index values are removed
                mux(j) = 0;         %Places unwanted cells that are too close at origin (0,0) - these will be removed below
                muy(j) = 0;
                index1 = index1 +1;
            end
        end
    end
    
    mundiffs = [mux' muy'];
    clear mux, muy;
    
    % Sort so that the zeros are first, keeping respective theta and radius values paired correctly
    % Then use 'ikill', and 'notkill' to cut out cells that were moved to (0,0)
    notkill = (length(ikill)+1);   %This is the index value of the first non zero value of the undiff
                                    %matrix, when arranged in ascending order... next 2 lines of code..-->
    mundiffs = sortrows(mundiffs, 2);     %sorts undiff matrix with zeros first
    Ux  = mundiffs(notkill:length(mundiffs),1);
    Uy  = mundiffs(notkill:length(mundiffs),2);
    %%%%%%%%%%%%%%%%% Done removing closely spaced undiff'ed cells
    
    %Convert differentiated cells to CARTESIAN COORDS before 'growth'
    Dx = ((diffAm(:,2)).*cos(diffAm(:,1)));
    Dy = ((diffAm(:,2)).*sin(diffAm(:,1)));
    
    numdiffAm  = length(diffAm);        % This finds the number of differentiated cells and 
    numUndiff = length(Ux);             % the number of precursor cells
    
    % Ready to "grow"
    lambdaI = NND * lambdaI_over_NND;   % Sets the value of the decaying signal constant to be used in growth.m
    
    growth; % Call the function GROWTH.M - this determines which cells differentiate in one phase of 'growth'
    
    %%%%%%%%%%%%% PLOT GROWTH AS IT PROCEEDS, LOOP BY LOOP %%%%%%%%%%%
    figure   
    plot(Dx(1:num_seed),Dy(1:num_seed),'o')
    hold on
    plot(Dx(num_seed+1:length(Dx)),Dy(num_seed+1:length(Dx)),'co')
    plot(Dx(num_seed+1:length(Dx)),Dy(num_seed+1:length(Dx)),'c.')
    axis equal tight
    
%    countDx(loop) = length(Dx((num_seed+1:length(Dx))));
    
end %%%%%%%% end of retinal growth loops

% plot out final pattern of differentiated cells
figure(1000)
polar(Dtheta, Drho,'o')
hold on
polar(Dtheta(num_seed+1:length(Dtheta)),Drho(num_seed+1:length(Dtheta)),'co')
polar(Dtheta(num_seed+1:length(Dtheta)),Drho(num_seed+1:length(Dtheta)),'c.') % add a dot to the middle of 'new' cells
title([ '(' filename ') Lambda_I= ' num2str(lambdaI) 'Threshold = ' num2str(threshold)]);
axis equal tight

%%%%%%%% SAVE COORDINATES OF SEED AND MODEL CELLS %%%%%%%%%%%%%%%%%%%%
%    these new cells will be analyzed below for NND, DRP, quadrat, etc.

Dsave_model = [Dx((num_seed+1):length(Dx)) Dy((num_seed+1):length(Dy))]; % write out only the 'new' differentiated cells (exclude the original seed)
save mod_amacrine Dsave_model -ascii

clear Tmp;

Dsave_data = [Dx(1:num_seed) Dy(1:num_seed)]; % write out only the 'data' cells (the original seed - re-centered)
save data_amacrine Dsave_data -ascii

save tmp1 lambdaI threshold filename
clear all % clear out variables before going on to analyses (some variable names may be used differently below)
load tmp1

%%%%%%%%%%%%%%%%%% SPATIAL STATISTICAL ANALYSES %%%%%%%%%%%%%%%%%%%%%
%  Now - do quantitative analyses of seed and of new model pattern

% Compare Model to Data  ***********
rplot = 1;  % initialize this so program won't get hung up if N < 10 (i.e. if this value isn't set in DRP, it can't be cleared)
v_drp_data = [1 1 1 1]; % ditto
for ianal = 1:2  % analyze data then model
    % Read in cell type 1 into arrays....
    if(ianal == 1)
        %    eval(['!copy ' files(filenum,:) ' jojo']); % analyze data
        eval(['!copy data_amacrine read_filename']); % analyze data - use pattern that was re-centered above
    else
        eval(['!copy mod_amacrine read_filename']); % analyze model
    end
    
    load read_filename;
    
    count1 = 0;
    count2 = 0;
    for(i = 1:length(read_filename(:,1)))
        count1 = count1 + 1;
        x1array(count1) = read_filename(i,1);
        y1array(count1) = read_filename(i,2);
    end
    x = x1array;
    y = y1array;
    
    n = length(x); % total # of cells
    
    %Find and plot analysis areas - code taken from the old DRP program
    % A = area of analysis
    if ianal == 1   % force same box size for model and data analysis
        rx = 0.9 * (max(x) - min(x));   % 0.9 'trims' off edges of seed pattern, which helps if pattern is 'rough' at edges.
        ry = 0.9 * (max(y) - min(y));    % if image has very irregular edges, adjust this value until analysis window is completely inside the image
    end    
    elx = rx;                         
    ely = ry;
    A = elx * ely;
    
    % find middle of pattern, and plot boundary of analysis rectangle on top of pattern
    xmid = (max(x) - min(x))/2 + min(x);  
    ymid = (max(y) - min(y))/2 + min(y); 
      
    %Find middle of y-values near x=0 to avoid problems with box placement
    y_min = 100000;
    for idumb = 1:length(y)
        if abs(x(idumb)) < (0.20 * (max(x) - xmid))
            if (y(idumb) < y_min) y_min = y(idumb); end
        end
    end
    %ymid = (max(y) - y_min)/2 + y_min;       
    
    xlo = xmid - elx/2; xhi = xmid + elx/2;  
    ylo = ymid - ely/2; yhi = ymid + ely/2;
    
    x1 = xlo; x2 = xhi;
    y1 = ylo; y2 = yhi;
    
    %plot area of analysis on top of final growth pattern
    figure(1000)
    plot([x1,x2],[y1,y1],'k')
    plot([x2,x2],[y1,y2],'k')
    plot([x2,x1],[y2,y2],'k')
    plot([x1,x1],[y2,y1],'k')
    
    %************* Find NND *********
    N = 0;
    for i = 1:n % go through cells in analysis box (this avoids edge effects for model data)
        if( (x(i) <= xhi) & (x(i) >= xlo) & (y(i) <= yhi) & (y(i) >= ylo) )
            N = N+1;
            dist = sqrt(((x - x(i)) .^ 2) + ((y - y(i)) .^ 2));
            dist = sort(dist);
            nnds(N) = dist(2); % skip 'mimimum' distance of 0 (distance to self)
            xref(N) = x(i);  % this array is used in DRP analysis below
            yref(N)= y(i);   %     ditto
        end
    end
    % Now, N = # of cells in the box - this is used for Quadrat analysis below
    ND = mean(nnds);
    NearestNeighbordist = sort(nnds)';
    NDten = ND * 10;            %the next 3 lines set the NND value to 1 decimal place
    NDtenround = round(NDten);
    NDfinal = NDtenround/10;
    StdDevNND = std(nnds);
    NumberCells = length(nnds);
    
    figure(1002)
    orient tall
    subplot(3,2,ianal); 
    if ianal == 1
        xhist = 0:(ND/10):(3 * ND); %use this set of bins for plotting both model and data
    end
    hist(nnds,xhist)
   if ianal == 1
       v_nnd_data = axis; % grab this value to use for model plot as well
       v_nnd_data(1) = 0.;
       v_nnd_data(2) = 4 * ND;
   end
   axis(v_nnd_data);
    xlabel('NND (micrometers)','Fontsize', 10)
    ylabel('No. cells','Fontsize', 10)
    if ianal == 1
        title(['Seed Mean NND = ' num2str(NDfinal)], 'Fontsize', 10);
        
    else
        title(['Model Mean NND= ' num2str(NDfinal)], 'Fontsize', 10);
    end
    %*******************************
    
    % ******* DO QUADRAT ANALYSIS **********
    A = elx * ely;
    D = N/A; % density of points in the region A
    
    ICS = zeros(1,10); % for max divisor = 10
    for divisor = 2:10
        
        %boxwidth = large_boxwidth/divisor;
        boxwidth = elx/divisor;
        boxheight = ely/divisor;
        num_boxes = divisor ^2;
        Nbox = zeros(1,num_boxes);
        
        for ibox = 1:num_boxes
            % define small box edges
            x1 = (mod((ibox-1),divisor))* boxwidth + xmid - elx/2;
            x2 = (mod((ibox-1),divisor)+1) * boxwidth + xmid - elx/2;
            y1 = floor((ibox-.001)/divisor)* boxheight + ymid - ely/2;
            y2 = (floor((ibox-.001)/divisor)+1) * boxheight + ymid - ely/2;
            % Now count the cells in this box.
            for i=1:n       
                if((x(i) <= x2) & (x(i) > x1) & (y(i) <= y2) & (y(i) > y1))  
                    Nbox(ibox) = Nbox(ibox) + 1;
                end
            end
        end
        
        %ICS = Index of Clumping in Ripley's book Spatial Statistics, pg. 104).
        ICS(divisor) = var(Nbox)/mean(Nbox) - 1;
        
        %"index" of dispersion - this is just the (var/mean) * (N-1)
        %  [Skellam 1952, David & Moore 1954]
        
        test(divisor) = var(Nbox) * (num_boxes - 1)/ mean(Nbox);
    end % end of loop through divisors
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % Plot quadrat results
    figure(1002)
    subplot(3,2,2+ianal)
    plot((2:divisor).^2,test(2:divisor),'k')
    if(ianal == 1)
        title(['Quadrat Analysis'],'Fontsize', 10)
    else
        title(['Model Quadrat Analysis'],'Fontsize', 10)
    end    
    
    xlabel('No. boxes','Fontsize', 10)
    ylabel('(Var/Mean)*(N-1)','Fontsize', 10)
    
    hold on
    plot([4 9 16 25 31 41 51 61 71 81 91 101],[7.81 15.51 25.0 36.42 43.77 55.76 67.5 79.08 90.53 101.88 113.15 124.34],'k.')
    % Significantly clumped (aggregate, or contagious) occurs when "test"=mean/var*(N-1)
    % exceeds the value of a chi^2 distribution for N-1 degreees of freedom, say, p < .05 (95% probability that pattern is not random).
    % Significantly regular patterns occur when the "test" value is _less than_ the
    % chi^2 distribution for N-1 degrees of freedom, for say, probability of 95% p > .95 (5% probability that pattern is random).
    hold on
    plot([4 9 16 25 31 41 51 61 71 81 91 101],[0.352 2.733 7.261 13.848 18.493 26.509 34.764 43.188 51.739 60.391 69.126 77.929],'k.')
    V = axis;
    axis([0 100 V(3) V(4)]);
    %%%%%%%%%%%% end of quadrat analysis
    
    %%%%%%%%% Do DRP ANALYSIS %%%%%%%%%%%
    % Analysis box was determined above....use same box
    
    if (N > 0) % only do density analysis when there are more than 5 cells within A (N > 5)   %  ->  5 was arbitrarily chosen
        
        num_annuli = 10.;
        delr = (elx/2.)/num_annuli;  % set up annuli with max annulus equal to space outside el^2
        
        % the following things can be computed outside iref loop.
        for index = 1:num_annuli
            area(index) = pi * (delr^2) * (2 * index - 1); % area of the annulus
            lambda(index) = N * D * area(index);   % expected # pts in annulus	
        end
        
        for index = 1:num_annuli  % loop through the annuli
            npts(index) = 0;      
            for iref = 1:N  % loop through the cells in Area A
                % count up cells in annulus
                for i=1:n   % loop through all cells in data set
                    temp_dist = sqrt( (x(i) - xref(iref))^2 + (y(i) - yref(iref))^2); 
                    if (temp_dist > (index - 1)*delr ) & (temp_dist <= index*delr )
                        npts(index) = npts(index) + 1;
                    end
                end
            end % end of iref loop - (counts through N reference cells)
            density(index) = npts(index) / (N * area(index));
        end % end of  index loop (index for annuli)
        
        % Calculate effective radius
        Ve = 0.;
        index = 1;
        while (npts(index) < lambda(index)) & (index < num_annuli)
            Ve = Ve + (lambda(index) - npts(index));
            index = index + 1;
        end
        Ve = Ve/N;
        re = sqrt(Ve/(pi * D));
        Density = D*1e6;
        Densityround = round(Density);
        figure(1002)
        subplot(3,2,4+ianal)
        if ianal == 1
            rplot = (1:num_annuli) * delr;
        end
        bar(rplot,density * 1e6 ,'k') % convert density to cells/(mm^2)
         if ianal == 1
             v_drp_data = axis;
%             axis([v_drp_data(1),v_drp_data(2),v_drp_data(3),v_drp_data(3) + 2*(v_drp_data(4) - v_drp_data(3))]); % double the y-axis to allow room for model (higher densities)
         elseif ianal == 2
%             v_drp_model = axis;
%             axis([v_drp_model(1),v_drp_model(1) + (v_drp_data(2) - v_drp_data(1)),v_drp_model(3), v_drp_model(3) + 2.*(v_drp_data(4) - v_drp_data(3))]);
%             axis([v_drp_data(1),v_drp_data(2),v_drp_data(3),v_drp_data(3) + 2*(v_drp_data(4) - v_drp_data(3))]); % Use same axis as for data
                axis(v_drp_data);
        end
        hold on
        plot([0 num_annuli * delr],[D*1e6 D*1e6],'k:')   %convert density to 1/(mm^2)
        plot([re re],[0 D*1e6],'k--') % plot effective radius
        xlabel('Distance from origin cell (micrometers)','Fontsize', 10)
        ylabel('Cell count (cells/mm^2)','Fontsize', 10)
        if ianal == 1
            title(['Density (cells/mm^2)= ' num2str(Densityround)],'Fontsize', 10);
        else
            title(['Density (cells/mm^2)= ' num2str(Densityround)],'Fontsize', 10);

        end
    end % end of N < 10 loop
end % end of ianal loop
